// CTESerial.cpp: implementation of the CTESerial class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TESerial.h"
#include <stdio.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTESerial::CTESerial()
{
	hPort= INVALID_HANDLE_VALUE;


}

CTESerial::~CTESerial()
{
	CloseComm();

}

// Main transmit function.  Pass in a CString of the text to send.
// If sucessful, it will return a 1.  Errors return 0
int CTESerial::Send(const CString &data)
{
	static DWORD dwWrite;

	if (hPort != INVALID_HANDLE_VALUE)
	{
		if (Write_Comm(data, data.GetLength(), dwWrite) || dwWrite != static_cast<DWORD>(data.GetLength()))
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
		return 0;
	}
}

// Main receive function.  Pass in a CString by reference that will contain the received data.
// If sucessful, it will return a 1.  Errors return 0
int CTESerial::Receive(CString &data)
{
	static DWORD dwRead;
	
	data.Empty();

	if (hPort != INVALID_HANDLE_VALUE)
	{
		Read_Comm(data, dwRead, NULL, true, '\r');	// Wait for a timeout, or a newline char
		if (dwRead > 0)
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
		return 0;
	}
}

// Clear out the comm buffer
void CTESerial::Purge_Comm()
{
	PurgeComm(hPort, PURGE_RXABORT | PURGE_RXCLEAR);


}

// Worker read routine.  This does the checking for last characters and timeouts.
BOOL CTESerial::Read_Comm(CString &str, DWORD &dwRead, const DWORD maxLen, bool lookforchar, char lastchar)
{
	// Clear out the string
	//str.Empty();
	BYTE CharBuf[2];
	CharBuf[1] = '\0';
	int cResp = TRUE;
	DWORD ReadData = 0;
	bool FoundChar = false;

	if (hPort == INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	if (maxLen == 0)
	{
		// No Max Length set.  Read until a Timeout or until we receive the terminating 
		//	char specified in lastchar
		while (1)
		{
			cResp = ReadFile(hPort, &CharBuf, 1, &dwRead, NULL);
			
			if (cResp == TRUE)
			{
				if (dwRead != 0)
				{
					str += CharBuf;

					ReadData += dwRead;
					if (lookforchar)
					{
						if (CharBuf[0] == (unsigned char) lastchar)
						{
							//OutputDebugString(_T("Found Terminating Character\r\n\0"));
							FoundChar = true;
							// We've found the last char.  break & return and leave the buffer intact
							break;
						}
					}	
					CharBuf[0] = '\0';
				}
				else
				{
					// Read Timeout occured
					break;
				}
			}
			else
				break;
		}
	}
	else
	{
		for(UINT count = 0; count < maxLen; count ++)
		{
			//cResp = ReadFile(hPort, (void *)((LPCTSTR ) str), 1, &dwRead, NULL);
			cResp = ReadFile(hPort, &CharBuf, 1, &dwRead, NULL);
			
			if (cResp == TRUE)
			{
				if (dwRead != 0)
				{
					// Read in valid data.  Add to the buffer
					str += CharBuf;

					ReadData += dwRead;
					if (lookforchar)
					{
						if (CharBuf[0] == lastchar)
						{
							// We've found the last char.  break & return and leave the buffer intact
							FoundChar = true;
							break;
						}
					}	
					CharBuf[0] = '\0';
				}
				else
				{
					// Read Timeout occured
					break;
				}
			}
		}
	}

	if (ReadData > 0 || cResp == TRUE)
	{
		dwRead = ReadData;
		if (lookforchar == true)
		{
			if (FoundChar == true)
			{
				return TRUE;
			}
			else
			{
				return FALSE;
			}
		}

		return TRUE;

	}
	
	return FALSE;

}

// Transmit worker routine
BOOL CTESerial::Write_Comm(const CString &str, const DWORD writeLen, DWORD &dwWrite)
{
	Purge_Comm();

	if (WriteFile(hPort, str, str.GetLength(), &dwWrite, NULL))
	{
		// should be written successfully
		if (writeLen == dwWrite)
		{
			return TRUE;
		}
	}
	return FALSE;

}

BOOL CTESerial::CloseComm()
{
	if (CloseHandle(hPort))
	{
		hPort = INVALID_HANDLE_VALUE;
		return TRUE;
	}

	return FALSE;

}

BOOL CTESerial::SetTimeouts(DWORD IntervalTimeout, DWORD IntervalMultiplier, DWORD IntervalAdjustment, DWORD WriteTimeout, DWORD WriteAdjustment)
{

	m_Commtimeouts.ReadIntervalTimeout = IntervalTimeout;
	m_Commtimeouts.ReadTotalTimeoutMultiplier = IntervalMultiplier;
	m_Commtimeouts.ReadTotalTimeoutConstant = IntervalAdjustment;
	m_Commtimeouts.WriteTotalTimeoutMultiplier = WriteTimeout;
	m_Commtimeouts.WriteTotalTimeoutConstant = WriteAdjustment;

	return SetCommTimeouts(hPort, &m_Commtimeouts);
	
}


int CTESerial::ShutDown()
{
	CloseComm();

	return 1;
}

///////////////////////////////
//	BOOL InitComm(UINT portnum, UINT baud, UINT parity, UINT databits, UINT stopbits
//
//  portnum = 1-4												(1 Default)
//	baud = 1200 - 115200										(57600 Default)
//  parity = PARITY_NONE, PARITY_EVEN, PARITY_ODD				(PARITY_NONE Default)
//  databits = DATABITS_7, DATABITS_8							(DATABITS_8 Default)
//	stopbits = STOPBITS_10, STOPBITS_15, STOPBITS_20			(STOPBITS_10 Default)
///////////////////////////////
BOOL CTESerial::InitComm(UINT portnum, UINT baud, UINT parity, UINT databits, UINT stopbits)
{
	BYTE bParity;
	BYTE bStopBits;

	// Range Checks on data here
	//////////////////////////////////////////////////////////////////
	if (portnum < 1 || portnum > 4)
	{
		return FALSE;
	}

	// Do a quick check of the baud here
	//////////////////////////////////////////////////////////////////
	if (baud < 1200 || baud >115200)
	{
		return FALSE;
	}


	// Check and set Parity here
	//////////////////////////////////////////////////////////////////
	if (parity == PARITY_NONE)
		bParity = 0;
	else if (parity == PARITY_ODD)
		bParity = 1;
	else if (parity == PARITY_EVEN)
		bParity = 2;
	else
		return FALSE;


	// Check and set StopBits here
	//////////////////////////////////////////////////////////////////
	if (stopbits == STOPBITS_10)
		bStopBits = 0;
	else if (stopbits == STOPBITS_15)
		bStopBits = 1;
	else if (stopbits == STOPBITS_20)
		bStopBits = 2;
	else
		return FALSE;

	// Check and set Databits here
	//////////////////////////////////////////////////////////////////
	if (databits == DATABITS_5)
		databits = 5;
	else if (databits == DATABITS_6)
		databits = 6;
	else if (databits == DATABITS_7)
		databits = 7;
	else if (databits == DATABITS_8)
		databits = 8;
	else
		return FALSE;

	// End Checks on data
	//////////////////////////////////////////////////////////////////

	m_csPort.Empty();
	m_csPort.Format("COM%i:", portnum);

	// Open the comm port
	hPort = CreateFile(m_csPort, GENERIC_READ | GENERIC_WRITE, 0, NULL, /*OPEN_ALWAYS*/ OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if (hPort != INVALID_HANDLE_VALUE)
	{
		// Opened the port.  Now set it up
		m_DCB.DCBlength = sizeof(DCB);
		if (GetCommState(hPort, &m_DCB))
		{
			m_DCB.BaudRate = baud;
			m_DCB.StopBits = bStopBits;
			m_DCB.Parity = bParity;
			m_DCB.ByteSize = (BYTE) databits;
			m_DCB.fDtrControl = DTR_CONTROL_DISABLE;

			if (!SetCommState(hPort, &m_DCB))
			{
				return FALSE;
			}

			// Set a max timeout when waiting for data.  Queries will only wait this long
			// if a carriage return is not found
			SetTimeouts(MAXDWORD, MAXDWORD, 1000);

			// Clear out the buffer before using
			Purge_Comm();

			return TRUE;
		}
		else
		{
			return FALSE;
		}

	}

	return FALSE;

}
